package grpc

import (
	"context"
	"crypto/tls"

	"git.nobla.cn/golang/aeus/pkg/logger"
	"git.nobla.cn/golang/aeus/registry"
	"google.golang.org/grpc"
)

const (
	Protocol = "grpc"
)

type (
	Option func(*options)

	options struct {
		network  string
		address  string
		grpcOpts []grpc.ServerOption
		logger   logger.Logger
		context  context.Context
	}

	clientOptions struct {
		tls         *tls.Config
		registry    registry.Registry
		dialOptions []grpc.DialOption
	}

	ClientOption func(*clientOptions)

	requestValueContextKey struct{}
)

func WithNetwork(network string) Option {
	return func(o *options) {
		o.network = network
	}
}

func WithAddress(address string) Option {
	return func(o *options) {
		o.address = address
	}
}

func WithContext(ctx context.Context) Option {
	return func(o *options) {
		o.context = ctx
	}
}

func WithLogger(lg logger.Logger) Option {
	return func(o *options) {
		o.logger = lg
	}
}

func WithTLS(tls *tls.Config) ClientOption {
	return func(o *clientOptions) {
		o.tls = tls
	}
}

func WithRegistry(reg registry.Registry) ClientOption {
	return func(o *clientOptions) {
		o.registry = reg
	}
}

func WithGrpcDialOptions(opts ...grpc.DialOption) ClientOption {
	return func(o *clientOptions) {
		o.dialOptions = opts
	}
}

func GetRequestValueFromContext(ctx context.Context) any {
	if ctx == nil {
		return nil
	}
	return ctx.Value(requestValueContextKey{})
}
