package http

import (
	"context"
	"net/http"

	"git.nobla.cn/golang/aeus/pkg/logger"
	"github.com/gin-gonic/gin"
)

const (
	Protocol = "http"
)

type (
	Option func(*options)

	options struct {
		network    string
		address    string
		certFile   string
		keyFile    string
		debug      bool
		hander     http.Handler
		logger     logger.Logger
		context    context.Context
		ginOptions []gin.OptionFunc
	}

	HandleFunc func(ctx *Context) (err error)

	Middleware func(http.Handler) http.Handler
)

func WithNetwork(network string) Option {
	return func(o *options) {
		o.network = network
	}
}

func WithAddress(address string) Option {
	return func(o *options) {
		o.address = address
	}
}

func WithCertFile(certFile string) Option {
	return func(o *options) {
		o.certFile = certFile
	}
}

func WithKeyFile(keyFile string) Option {
	return func(o *options) {
		o.keyFile = keyFile
	}
}

func WithLogger(lg logger.Logger) Option {
	return func(o *options) {
		o.logger = lg
	}
}

func WithContext(ctx context.Context) Option {
	return func(o *options) {
		o.context = ctx
	}
}

func WithDebug(debug bool) Option {
	return func(o *options) {
		o.debug = debug
	}
}

func WithHandler(h http.Handler) Option {
	return func(o *options) {
		o.hander = h
	}
}

func WithGinOptions(opts ...gin.OptionFunc) Option {
	return func(o *options) {
		o.ginOptions = opts
	}
}
